#!/bin/sh
#
#  Firewall Builder
#
#  Copyright (C) 2002 NetCitadel, LLC
#
#
#
# this script performs upgrade of Firewall Builder data file by running
# appropriate XSLT transformations sequentially until version becomes
# current.
#
#   Author: Vadim Kurland, <vadim@fwbuilder.org>
#

XMLFILE=$1
VERS=$2

if test -z "${XMLFILE}"; then
    echo "Usage: fwb-upgrade.sh file.xml [ version ]"
    echo "       upgrades file.xml to version 'version'"
    echo "       If 'version' is not specified, upgrades to the current version"
    echo "       of libfwbuilder installed on the system"
    exit 1
fi

XSLTPROC=`which xsltproc`
if test -z "${XSLTPROC}"; then
 echo "Can't find xsltproc, check if you have libxslt installed"
 exit 1
fi



INST_DIR="/usr/share/libfwbuilder"
TMP1=".fwb.upgr.xml"
FILE=$XMLFILE

cp ${FILE} ${FILE}.bak

LIBFWBUILDER_CONFIG=`which libfwbuilder-config 2>/dev/null`
if test -n "${LIBFWBUILDER_CONFIG}"; then
  CURRENT_VERSION=$(${LIBFWBUILDER_CONFIG} --version)
  INST_DIR=$(${LIBFWBUILDER_CONFIG} --templatedir)
else
  DTDFILE=${INST_DIR}/fwbuilder.dtd
  CURRENT_VERSION=$(sed -n '/FWObjectDatabase/,/version/s/.*version[^"]*"//p' ${DTDFILE} | sed 's/"//')
fi

if test -n "$VERS"; then
  CURRENT_VERSION=$VERS
fi

while :;
do
  FILE_VERSION=$(echo "
<xsl:stylesheet version=\"1.0\" xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">
<xsl:output method=\"text\"/>
<xsl:strip-space elements=\"*\"/>
<xsl:template match=\"*[attribute::id='root']\">
<xsl:value-of select=\"@version\"/>
</xsl:template>
</xsl:stylesheet>
" | ${XSLTPROC} --novalid - $XMLFILE 2>/dev/null)

  if test "$FILE_VERSION" = "$CURRENT_VERSION"; then
    break
  fi

  TRANSFORMATION="${INST_DIR}/migration/${FILE_VERSION}/FWObjectDatabase.xslt"

  if test -f ${TRANSFORMATION}; then
    echo ${FILE_VERSION}

#    cp $FILE ${FILE_VERSION}.xml

    ${XSLTPROC} ${TRANSFORMATION} $FILE > $TMP1 2>/dev/null  || {
      echo "Error running xsltproc"
      exit 1
    }

    mv $TMP1 $FILE
  else
    echo "Missing transformation script ${TRANSFORMATION}"
    exit 1
  fi
done

echo "Fixing up"

#
# This transofrmation fixes a bug in one of the upgrade transform scripts
# which sticks interface in front of NAT in the Firewall element
#
echo "
<xsl:stylesheet version=\"1.0\"
    xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"
    xmlns:fwb=\"http://www.fwbuilder.org/1.0/\"
    exclude-result-prefixes=\"fwb\">
<xsl:output method=\"xml\" version=\"1.0\" 
   doctype-system=\"fwbuilder.dtd\" indent=\"yes\" encoding=\"utf-8\"/>
<xsl:template name=\"copy-element\">
  <xsl:copy>
    <xsl:for-each select=\"@*\">
      <xsl:copy/>
    </xsl:for-each>
    <xsl:apply-templates mode=\"copy\"/>
  </xsl:copy>
</xsl:template>
<xsl:template match=\"fwb:*\" mode=\"copy\">
  <xsl:choose>
    <xsl:when test=\"name()='Firewall'\">
      <xsl:call-template name=\"firewall\"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name=\"copy-element\"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>
<xsl:template name=\"firewall\">
  <xsl:copy>
    <xsl:for-each select=\"@*\">
      <xsl:copy/>
    </xsl:for-each>
    <xsl:for-each select=\"fwb:NAT\">
      <xsl:call-template name=\"copy-element\"/>
    </xsl:for-each>
    <xsl:for-each select=\"fwb:Policy\">
      <xsl:call-template name=\"copy-element\"/>
    </xsl:for-each>
    <xsl:for-each select=\"fwb:Interface\">
      <xsl:call-template name=\"copy-element\"/>
    </xsl:for-each>
    <xsl:for-each select=\"fwb:Management\">
      <xsl:call-template name=\"copy-element\"/>
    </xsl:for-each>
    <xsl:for-each select=\"fwb:FirewallOptions\">
      <xsl:call-template name=\"copy-element\"/>
    </xsl:for-each>
  </xsl:copy>
</xsl:template>
<xsl:template match=\"*[attribute::id='root']\">
  <FWObjectDatabase xmlns=\"http://www.fwbuilder.org/1.0/\">
    <xsl:for-each select=\"@*\">
      <xsl:copy/>
    </xsl:for-each>
  <xsl:apply-templates select=\"*\" mode=\"copy\"/>
  </FWObjectDatabase>
</xsl:template>
</xsl:stylesheet>
" | ${XSLTPROC} --novalid - $XMLFILE  > $TMP1 2>/dev/null  || {
  echo "Error running xsltproc"
  exit 1
}

mv $TMP1 $XMLFILE

echo '*******************************************************'
echo
echo "File $XMLFILE has been upgraded, current version is $CURRENT_VERSION"
echo
echo "Backup copy is in ${XMLFILE}.bak"
echo

